/*
 * dpkg - main program for package management
 * declactions.c - declarative package actions
 *
 * Copyright © 2007, 2011-2015 Guillem Jover <guillem@debian.org>
 * Copyright © 2010 Raphaël Hertzog <hertzog@debian.org>
 * Copyright © 2008 Joey Hess <joeyh@debian.org>
 * Copyright © 2005 Scott James Remnant (original implementation on www.dpkg.org)
 * Copyright © 2020 Niels Thykier <niels@thykier.net>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <sys/types.h>
#include <sys/stat.h>

#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <dpkg/dpkg.h>
#include <dpkg/varbuf.h>

#define DPKGREMOVEEXT ".dpkg-remove"
#define DPKGBACKUPEXT ".dpkg-backup"
#define DPKGBAKEXT ".dpkg-bak"

/*
finish_rm_conffile() {
	local CONFFILE="$1"

	if [ -e "$CONFFILE.dpkg-backup" ]; then
		echo "Obsolete conffile $CONFFILE has been modified by you."
		echo "Saving as $CONFFILE.dpkg-bak ..."
		mv -f "$CONFFILE.dpkg-backup" "$CONFFILE.dpkg-bak"
	fi
	if [ -e "$CONFFILE.dpkg-remove" ]; then
		echo "Removing obsolete conffile $CONFFILE ..."
		rm -f "$CONFFILE.dpkg-remove"
	fi
}*/

/***

──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
19:53 < guillem> right, my point probably too subtle, is that this might require declarative support
19:54 < guillem> so either the mtree stuff, or extending the format of the conffiles file
19:54 < guillem> but that might require waiting one rel cycle anyway :/
19:54 < nthykier> my prototype would be based on the latter
19:54 < nthykier> you are in the deploment phase already and I do not even have typed in a single line of code yet! :P
19:55 < nthykier> can I assume that means you have it fixed in a branch and it will be merged soon? ;)
19:55 < guillem> Ah I already mentioned the conffiles extensions in the bug :D
19:55 < guillem> nthykier: I don't think so! or at least don't remember so! :)
20:09 < guillem> nthykier: ok, so I guess the tricky part here is the move and rename cases, which would need way more logic
20:09 < guillem> handling just the remove case might not be too difficult I guess
20:11 < guillem> nthykier: you'd need to parse the new keyword in the Conffiles field, in check_conffiles() and in deb_parse_conffiles()
20:12 < guillem> then I guess probably also add a new FNNF flag for the marked removal
20:13 < guillem> and I guess another bool to struct conffile
20:13 < guillem> (but I've not really thought any of this through :)
20:14 < guillem> you'd also need to update pkg_update_fields() to generate the conffiles structs
20:14 < guillem> and then hook the conffile removal during unpack somewhere :D
20:16 < guillem> hmm not much progress with https://bugs.debian.org/cgi-bin/pkgreport.cgi?tag=dpkg-db-access-blocker;users=debian-dpkg@lists.debian.org :/


20:53 < nthykier> mmm, how would rollback work if I delete the conffile before we are in the unpacked state?
20:54 < nthykier> I presume deletion has to be done at some point *after* unpacked (not entirely sure where)
20:55 < guillem> nthykier: my point is that we should be fine with the dpkg generated backup, no need for the extra one, but I've not thought how to integrate any of
                 those properly

20:56 < guillem> also the problem here is that conffile handling is pretty much deferred to --configure so it's all a bit weird
21:02 < guillem> nthykier: on unpack everything is unpacked as foo.dpkg-new, then the old files are backed up as foo.dpkg-tmp, and then the new file is renamed
                 atomically from foo.dpkg-new to foo
21:02 < guillem> for conffiles this is deferred to --configure


/// Looks interesting:
  deferred_configure_conffile

 ***/

static int file_exists(const char *path) {
    struct stat stab;
    if (stat(path, &stab)) {
        if (errno == ENOENT) {
                return 0;
        }
        ohshite(_("unable to stat '%.250s'"), path);
    }
    return 1;
}

void finish_rm_conffile(const char *conffile_path) {
    struct varbuf fnamedremove;
    struct varbuf fnamedbackup;

    varbuf_reset(&fnamedbackup);
    varbuf_add_str(&fnamedbackup, conffile_path);
    varbuf_add_str(&fnamedbackup, DPKGBACKUPEXT);
    varbuf_end_str(&fnamedbackup);


    if (file_exists(fnamedbackup.buf)) {
        struct varbuf fnamedbak;
        varbuf_reset(&fnamedbak);
        varbuf_add_str(&fnamedbak, conffile_path);
        varbuf_add_str(&fnamedbak, DPKGBAKEXT);
        varbuf_end_str(&fnamedbak);

        printf(_("Obsolete conffile %.250s has been modified by you.\n"),
            conffile_path);
        printf(_("Saving as %.250s ...\n"), fnamedbak.buf);
        if (rename(fnamedbackup.buf, fnamedbak.buf))
          ohshite(_("unable to rename obsolete conffile '%.250s' to '%.250s'"),
              fnamedbackup.buf, fnamedbak.buf);

        varbuf_destroy(&fnamedbak);
    }
    varbuf_destroy(&fnamedbackup);

    varbuf_reset(&fnamedremove);
    varbuf_add_str(&fnamedremove, conffile_path);
    varbuf_add_str(&fnamedremove, DPKGREMOVEEXT);
    varbuf_end_str(&fnamedremove);

    if (file_exists(fnamedremove.buf)) {
        printf(_("Removing obsolete conffile %.250s ...\n"),
            fnamedremove.buf);
	if (unlink(fnamedremove.buf))
		ohshite(_("cannot remove '%.250s'"), fnamedremove.buf);
    }
    varbuf_destroy(&fnamedremove);
}
